#!/usr/bin/python

#parse LaTeX code, extract verbatim code segment to an R script.

import getopt, sys

class LaTeXParser:
    CODEOUT="R"
    COMMENT = "# "  # comment prefix. Include a space.
    CHAPTER = r"\chapter{"
    CHAPTEREND="}"
    SECTION = r"\section{"
    SECTIONEND="}"
    SUBSECTION = r"\subsection{"
    SUBSECTIONEND = "}"
    FOOTNOTE=r"\footnote{"
    FOOTNOTEEND="}"
    SPACER="\n"

    CODEPAIRS = {}
    CODEPAIRS[r"\begin{verbcode}"] = r"\end{verbcode}"
    CODEPAIRS[r"\begin{verbrun}"] = r"\end{verbrun}"
    CODEPAIRS[r"\begin{BVerbatim}[baseline=t]"] = r"\end{BVerbatim}"

    CODESTARTS=CODEPAIRS.keys()

    def extractCode(self, infilename):
        """Read line by line, extract code segments, write out
        to file with comments before specifying section code was found in"""
        outfilename = infilename[:(len(infilename)-4)] + "." + self.CODEOUT
        infile = open(infilename, 'r')
        outfile = open(outfilename, 'w')
        textmap = {'A Brief Introduction to R':1, 'Styles of Data Analysis':2,\
'Statistical Models':3, 'An Introduction to Formal Inference':4,\
'Regression with a Single Predictor':5, 'Multiple Linear Regression':6,\
'Exploiting the Linear Model Framework':7,\
'Logistic Regression and Other Generalized Linear Models':8,\
'Multi-level Models, Time Series and Repeated Measures':9,\
'Tree-based Classification and Regression':10,\
'Multivariate Data Exploration and Discrimination':11,\
'The R System -- Additional Topics':12, 'Epilogue -- Models':13}
        currsection=""
        currsubsection=""
        oldcurrsection=""
        oldcurrsubsection=""
        currcodelines = []
        currcodestart=""
        currcodeend=""
        currmode=1
        secnum=0
        line = infile.readline()


        while line != "":
            # modes for parser:
            # 1 = looking for a start of section, subsection, footnote or code.
            # 2 = in a code section

            if currmode == 1:
                #print "in mode 1, reading line: "+ line
                if line.find(self.CHAPTER) != -1:
                    currchapter = line[line.find(self.CHAPTER) + len(self.CHAPTER):line.find(self.CHAPTEREND)]
                    chapnum = textmap[currchapter]
                    outfile.write(self.COMMENT + "CHAPTER %d: %s %s"\
 %(chapnum, currchapter, "\n"))
                if line.find(self.SECTION) != -1:
                    secnum = secnum+1
                    currsection = line[line.find(self.SECTION) + len(self.SECTION):line.find(self.SECTIONEND)]
                    #print "New section: " + currsection
                    currsubsection=""
                elif line.find(self.SUBSECTION) != -1:
                    currsubsection = line[line.find(self.SUBSECTION) + len(self.SUBSECTION):line.find(self.SUBSECTIONEND)]
                    #print "  subsection: " + currsubsection

                elif line.find(self.FOOTNOTE) != -1:
                    # not doing anything with this yet.
                    #print "found->" + self.FOOTNOTE + "<- in " + line
                    infootnote=1

                else:
                    #print "checking codepairs"
                    #print self.CODEPAIRS.keys()
                    for code in self.CODEPAIRS.keys():
                        if line.find(code) != -1:
                            currmode=2
                            currcodestart = code
                            currcodeend = self.CODEPAIRS[code]
                            if currsection != oldcurrsection or currsubsection != oldcurrsubsection:
                                if oldcurrsection != "": outfile.write("\n")
                                if currsection != oldcurrsection:
                                    outfile.write(self.COMMENT +\
"-- Section %d.%d%s %s %s"%(chapnum, secnum, ": ", currsection, " -- #\n"))
                                else:
                                    outfile.write(self.COMMENT +\
"[%d.%d%s %s%s" %(chapnum, secnum,"]\t", currsubsection, "\n"))
                                outfile.write(self.COMMENT + "\n")
                                oldcurrsection = currsection
                                oldcurrsubsection = currsubsection


            if currmode == 2:
                # appending code - possline is the possible line to add
                # once tags are dropped. Ignore if empty.

                #print "mode 2, poss code line: " + line
                begin = line.find(currcodestart)
                end = line.find(currcodeend)

                if begin == -1 and end == -1:
                    possline = line
                    eolchar = ""

                if begin != -1:
                    # Drop anything not needed
                    # let this test flow through in case begin and end
                    # are on the same one line
                    line =  line[begin+len(currcodestart):].strip()
                    possline = ""
                    if line != "" and end == -1:
                        possline = line
                        eolchar = "\n"

                if end != -1:
                    possline = line[:end].strip()
                    eolchar = "\n" + self.SPACER
                    outfile.write("\n")
                    currmode=1
                
                if currcodestart == r"\begin{verbrun}":
                    if possline[:2] == "> " or possline[:2] == "+ ":
                        possline = possline[2:]
                    else: 
                        possline = ""
                if possline != "":
                    outfile.write(possline + eolchar)

            line = infile.readline()



# test, runner etc
if __name__ ==  "__main__":

    usage="""Usage options:
    -h help
    -f filename"""

    infile = ""
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hf:", ["help", "file="])
    except getopt.GetoptError:
        # print help information and exit:
        print usage
        sys.exit(2)

    for o, a in opts:
        if o in ("-h", "--help"):
            print usage
            sys.exit()
        if o in ("-f", "--file"):
            infile = a

    if infile=="":
        print "Please specify input file"
        print usage
        sys.exit()

    lp = LaTeXParser()
    #infile = "inference.tex"
    lp.extractCode(infile)



